<?php
/**
 * Library hook helper
 *
 * @package "All In One Meta" library
 * @since   1.0.0
 * @version 1.0.0
 */

// Prevent direct script access.
if ( !defined( 'ABSPATH' ) ) {
	die( 'No direct script access allowed' );
}

if( ! class_exists( 'AIOM_Hooks' ) ) {

	/**
	 * Class AIOM_Hooks
	 */
	final class AIOM_Hooks {

		/**
		 * Holds class single instance
		 * @var null
		 */
		private static $_instance = NULL;

		/**
		 * Get instance
		 * @return AIOM_Hooks|null
		 */
		public static function get_instance() {

			if ( NULL == static::$_instance ) {
				static::$_instance = new self();
			}

			return static::$_instance;

		}

		/**
		 * AIOM_Hooks constructor.
		 */
		private function __construct() {
			$this->hooks();
		}

		/**
		 * A dummy magic method to prevent AIOM_Hooks from being cloned.
		 *
		 */
		public function __clone() {
			throw new Exception( 'Cloning ' . __CLASS__ . ' is forbidden' );
		}

		/**
		 * Setup Hooks
		 */
		public function hooks() {
			add_action( 'save_post', array( $this, 'aiom_save_post' ), 1, 2 );
			add_action( 'created_term', array( $this, 'aiom_save_term' ), 1, 3 );
			add_action( 'edit_term', array( $this, 'aiom_save_term' ), 1, 3 );
			add_action( 'personal_options_update', array( $this, 'aiom_save_user' ), 1, 1 );
			add_action( 'edit_user_profile_update', array( $this, 'aiom_save_user' ), 1, 1 );
		}

		/**
		 * Trigger prefixed 'aiom_save_post' action for internal usage on wordpress's core 'save_post' action
		 * @param int $post_id Post ID
		 * @param WP_Post $post Current post object
		 */
		public function aiom_save_post( $post_id, $post ) {
			if( ( isset( $_POST[ 'has_aiom_data' ] ) && $_POST[ 'has_aiom_data' ] ) && apply_filters( 'aiom_allow_save', true ) ) {
				do_action( 'aiom_save_post', $post_id, $post );
			}
		}

		/**
		 * Trigger prefixed 'aiom_save_term' action for internal usage on
		 * wordpress's core 'edit_term' or 'created_term' action
		 *
		 * @param int    $term_id  Term ID.
		 * @param int    $tt_id    Term taxonomy ID.
		 * @param string $taxonomy Taxonomy slug.
		 */
		public function aiom_save_term( $term_id, $tt_id, $taxonomy ) {
			if( ( isset( $_POST[ 'has_aiom_data' ] ) && $_POST[ 'has_aiom_data' ] ) && apply_filters( 'aiom_allow_save', true ) ) {
				do_action( 'aiom_save_term', $term_id, $taxonomy );
			}
		}

		/**
		 * Trigger prefixed 'aiom_save_user' action for internal usage on
		 * wordpress's core 'edit_term' or 'created_term' action
		 *
		 * @param int    $user_id  User ID.
		 */
		public function aiom_save_user( $user_id ) {
			if( ( isset( $_POST[ 'has_aiom_data' ] ) && $_POST[ 'has_aiom_data' ] ) && apply_filters( 'aiom_allow_save', true ) ) {
				do_action( 'aiom_save_user', $user_id );
			}
		}

	}

	AIOM_Hooks::get_instance();

}